# lomo.py  A Python-Fu script to turn a GIMP-image into a 'LOMO' snap.
#
# version 0.3
# 
# Changed the circular fall off mask to an elliptical one.  The code
# for this change is based on:
# goose24-photo-lomo.scm
# by Dave Seaton 
# http://thermos.goose24.org
#
# version 0.1
#
# This version was coded in Python so the Lomo camera effect could be 
# automated with gimp 2.4.
#
# This is my first attempt at Python-Fu, Any-Kind-Of-Fu or Python.  Any
# feedback would be much appreciated.  I'm sure that wading through the 
# large amount of information needed to write a Python-Fu script has led
# to coding that might be improved.
#
# Copyright (C)November 2007 under GPL by Jim Sabatke <jsabatke@gmail.com>
# 
# Original Script-Fu
# Copyright (C)March 2003 under GPL by Avi Bercovich <avi@sillypages.org>
#
# Credit also to van der Vossen:
# http://stuff.vandervossen.net/archive/weblog/2003/04/faking_lomo.html
#
# This program is free software# you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation# either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program# if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#0

from gimpfu import *

def python_lomo(timg, tdrawable, contrast, saturation, feather, corner_darkness):
    
    # first modify bottom (original) layer's colors

    saturation_hue_offset = 0.0
    lightness=0
    hues_offset = 0
    brightness=0
    multiply_position=1
    
    original_foreground=pdb.gimp_context_get_foreground()
    	    
    if contrast:
        pdb.gimp_brightness_contrast(tdrawable, brightness, contrast)
    if saturation:
        pdb.gimp_hue_saturation(tdrawable, ALL_HUES,hues_offset, lightness, saturation)
    
    width=tdrawable.width
    height=tdrawable.height
    wfeather = width/feather
    hfeather = wfeather
    
    # setup backgound layer
    img = gimp.Image(width, height, RGB)
    img.disable_undo()
    
    pdb.gimp_image_undo_group_start(timg)
    
    
    frame_layer=gimp.Layer(timg,"frame", width, height, RGB_IMAGE, 100, OVERLAY_MODE)
    timg.add_layer(frame_layer,BACKGROUND_FILL)
    pdb.gimp_image_set_active_layer(timg,frame_layer)
    #pdb.gimp_rect_select(timg,wfeather,hfeather,width-(wfeather*2),height-(hfeather*2),
    #                     CHANNEL_OP_ADD,True,feather)
    pdb.gimp_rect_select(timg,1,1,width-2,height-2,
                         CHANNEL_OP_ADD,True,width/12)
    pdb.gimp_selection_invert(timg)
    pdb.gimp_edit_bucket_fill(frame_layer, FG_BUCKET_FILL,NORMAL_MODE,100,
                              0,False,0,0)
    pdb.gimp_selection_invert(timg)
    pdb.gimp_edit_bucket_fill(frame_layer, BG_BUCKET_FILL,NORMAL_MODE,100,
                                  0,False,0,0)
    pdb.gimp_progress_update(.25)
    
    #frame2_layer = pdb.gimp_layer_copy(frame_layer,False)
    #pdb.gimp_image_add_layer(timg,frame2_layer,3)
    
    # pdb.gimp_image_set_active_layer(timg,tdrawable)
   
    # flare layer (ellipse)
    
    flare_layer=pdb.gimp_layer_new(timg, width, height, RGB_IMAGE, "corners",			                                    corner_darkness, OVERLAY_MODE)
    pdb.gimp_image_add_layer(timg,flare_layer,2)
    pdb.gimp_layer_add_alpha(flare_layer)
    pdb.gimp_drawable_fill(flare_layer,TRANSPARENT_FILL)
    
    feather_zone = (height+width)/corner_darkness
    percent_overrun=0.1
    pdb.gimp_ellipse_select(timg,  0- ( percent_overrun *(width/feather)*2),
				   0 - (percent_overrun * (height/feather)*2),
				   width+(width/feather)*2,
				   height + (height/feather)*2,
				       CHANNEL_OP_REPLACE, True, True,
				       feather_zone)


    pdb.gimp_progress_update(.50)
    
    pdb.gimp_selection_invert(timg)
    pdb.gimp_palette_set_foreground([0,0,0])
    pdb.gimp_bucket_fill(flare_layer, FG_BUCKET_FILL, NORMAL_MODE, 100, 255, 0, 1, 1)
    pdb.gimp_palette_set_foreground(original_foreground)
    pdb.gimp_selection_none(timg)
				       
   
    # create a multiply layer

    pdb.gimp_image_set_active_layer(timg,tdrawable)
    multiply_layer=pdb.gimp_layer_copy(tdrawable,False)
    pdb.gimp_image_add_layer(timg,multiply_layer,multiply_position)
    pdb.gimp_layer_set_mode(multiply_layer,MULTIPLY_MODE)
    pdb.gimp_layer_set_opacity(multiply_layer,80)
        
    pdb.gimp_image_raise_layer_to_top(timg,multiply_layer)
    pdb.gimp_image_raise_layer_to_top(timg,flare_layer)
    pdb.gimp_image_raise_layer_to_top(timg,frame_layer)
    
    pdb.gimp_progress_update(1.0)
 
  
    pdb.gimp_image_undo_group_end(timg)
    pdb.gimp_displays_flush()
    
    gimp.delete(img)
 
register(
    
        "python_fu_lomo",
        "Simulate a Lomo photo",
        "Simulate a Lomo photo",
        "James Sabatke",
        "James Sabatke",
        "2007",
        "<Image>/Python-Fu/Photo/_Lomo",
        "RGB*, GRAY*",
	[
		(PF_SPINNER, "contrast", "Contrast", 0, (-20,30,1)),
		(PF_SPINNER, "saturation", "Saturation", 0, (-20,30,1)),
		(PF_SPINNER,"feather","Border Shadow Width Inverse", 50, (25,100,1)),
		(PF_SPINNER,"corner_darkness", "Corner Darkness", 80, (10,100,1)),
        ],
	[],
        python_lomo)

main()